package com.drht.datamanage;

import com.drht.datamanage.handler.CheckDeviceInChannelHandler;
import com.drht.datamanage.handler.CheckMsgInChannelHandler;
import com.drht.datamanage.handler.ReplyDeviceInChannelHandler;
import com.drht.datamanage.handler.SendChannelHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.LineBasedFrameDecoder;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;
import io.netty.handler.timeout.ReadTimeoutHandler;
import lombok.extern.slf4j.Slf4j;

import static io.netty.channel.ChannelOption.ALLOCATOR;

/**
 *
 *
 */
@Slf4j
public class App {

    public static void run() {
        log.info("初始化netty");
        // accept线程组，用来接受连接
        EventLoopGroup bossGroup = new NioEventLoopGroup(1);

        // I/O线程组， 用于处理业务逻辑
        EventLoopGroup workerGroup = new NioEventLoopGroup(1);

        try {
            // 服务端启动引导
            ServerBootstrap b = new ServerBootstrap();

            b.group(bossGroup, workerGroup) // 绑定两个线程组
                    .channel(NioServerSocketChannel.class) // 指定通道类型
                    .option(ChannelOption.SO_BACKLOG, 10) // 设置TCP连接的缓冲区
                    .option(ChannelOption.RCVBUF_ALLOCATOR,new FixedRecvByteBufAllocator(1000*1024))
                    .handler(new LoggingHandler(LogLevel.INFO)) // 设置日志级别
                    .childHandler(
                            new ChannelInitializer<SocketChannel>() {
                                @Override
                                protected void initChannel(SocketChannel socketChannel) throws Exception {
                                    ChannelPipeline pipeline = socketChannel.pipeline(); // 获取处理器链
                                    pipeline.addLast(new LineBasedFrameDecoder(1000*1024)); // 读取第一步处理，功能是一次读一行
                                    pipeline.addLast(new StringDecoder()); // 转成字符串
                                    pipeline.addLast(new StringEncoder()); // 编码成二进制
                                    pipeline.addLast(new SendChannelHandler()); // 发送消息
                                    pipeline.addLast(new ReadTimeoutHandler(90)); // 超时处理,90秒未接收到数据认为设备离线
                                    pipeline.addLast(new CheckMsgInChannelHandler()); // 检查消息格式、crc校验等
                                    pipeline.addLast(new CheckDeviceInChannelHandler()); // 检查设备是否可用
                                    pipeline.addLast(new ReplyDeviceInChannelHandler()); // 回复消息
                                }
                            });

            // 通过bind启动服务
            ChannelFuture f = b.bind(7001).sync();
            // 阻塞主线程，知道网络服务被关闭
            f.channel().closeFuture().sync();

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            workerGroup.shutdownGracefully();
            bossGroup.shutdownGracefully();
        }
    }
}

