package com.drht.datamanage.handler;


import com.drht.datamanage.util.AnalysisUtil;
import com.drht.datamanage.util.SpringContextUtil;
import com.drht.datamanage.dao.HourlyDataDao;
import com.drht.datamanage.exception.AnalysisException;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.timeout.ReadTimeoutException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;

@Slf4j
public class CheckDeviceInChannelHandler extends SimpleChannelInboundHandler {
//    @Autowired
//    private DeviceMapper deviceMapper;

    // crc校验后调用检查设备是否可用
    @Override
    public void channelRead0(ChannelHandlerContext ctx, Object msg) throws Exception {
        ApplicationContext context = SpringContextUtil.context;
        HourlyDataDao deviceMapper = (HourlyDataDao) context.getBean("hourlyDataDao");
        //解析报文
        String s=(String)msg;
        AnalysisUtil.Msg msgo = AnalysisUtil.msgToJsonString(s);
        //验证设备
        String mn=msgo.getMN();
//        String status = deviceMapper.queryStatusByMn(mn);
//        if (!"0".equals(status)){
//            log.info("验证mn：{}的设备没通过，msg:{}",mn,"设备不在线!");
//            ctx.close();
//            return;
//        }
        //向下传递
        ctx.fireChannelRead(msgo);
    }

    // 数据读取完后被调用
    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
        ctx.flush();
    }

    // 当Netty由于IO错误或者处理器在处理事件时抛出的异常时被调用
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        if (cause instanceof ReadTimeoutException){
            //超时异常
            log.info("关闭了一个90s内没有发送任何数据的链接");
            ctx.close();

        }else if (cause instanceof AnalysisException){
            AnalysisException e = (AnalysisException) cause;
            //解析报文出错的异常
            log.info("解析报文出错：code={}，msg={}",e.getCode(),e.getMsg());
            ctx.close();
            //
        }else{
            //其他异常向下传递
            ctx.fireExceptionCaught(cause);
        }

    }
}
