package com.drht.datamanage.handler;

import com.drht.datamanage.util.HelperUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class CheckMsgInChannelHandler extends SimpleChannelInboundHandler {

    // 检查消息合法否，crc校验
    @Override
    public void channelRead0(ChannelHandlerContext ctx, Object msg) throws Exception {
        String str=(String)msg;
        log.info("原始数据{}",str);
        if (!(str.contains("QN")
                &&str.contains("PW")
                &&str.contains("MN")
                &&str.contains("CN")
                &&str.contains("CP"))){
//            不合法的包，关闭通道，记录
            log.info("不合法{}，通道关闭",str);
            ctx.close();
            return;
        }
        if (!str.startsWith("##")){
            //包头不是##开头，记录日志关闭通道
            log.info("不合法{}，不是##开头，通道关闭",str);
            ctx.close();
            return;
        }
        //获取报文crc
        String crc=str.substring(str.length()-4);
        //截取掉包头与包未
        str=str.substring(2,str.length()-4);

        //crc校验
        int checkCrc = HelperUtil.getCheckCrc(str.getBytes());
        //转16进制并转成大写
        String scheckCrc = String.format("%04x",checkCrc).toUpperCase();
//        if (!(scheckCrc.equals(crc.toUpperCase()))){
//            //未通过crc校验
//            log.info("不合法{}，未通过crc校验，报文crc{}，服务端生成的crc{}，通道关闭",str,crc,scheckCrc);
//            ctx.close();
//            return;
//        }
        log.info("第一层校验处理后{}",str);
        ctx.fireChannelRead(str);

    }

    // 数据读取完后被调用
    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
        ctx.flush();
    }

    // 当Netty由于IO错误或者处理器在处理事件时抛出的异常时被调用
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        //到下一个handler处理
            ctx.fireExceptionCaught(cause);

    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        if (ctx.channel().remoteAddress().toString().startsWith("/100.")) {
            ctx.close();
            return;
        }
        super.channelActive(ctx);

    }



}
