package com.drht.datamanage.util;

import cn.hutool.core.map.MapUtil;
import com.alibaba.fastjson.annotation.JSONField;
import com.drht.datamanage.exception.AnalysisException;
import com.drht.datamanage.model.msg.SendMsg;
import com.google.common.collect.Lists;
import io.netty.util.internal.logging.InternalLogger;
import io.netty.util.internal.logging.InternalLoggerFactory;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

public class AnalysisUtil {
    private static final InternalLogger logger = InternalLoggerFactory.getInstance(AnalysisUtil.class);

    public static AnalysisUtil.Msg msgToJsonString(String msg) throws AnalysisException {
        //报文对象
        Msg m=new Msg();
        //cp对象
        Map cpMap=new HashMap();
        //验证cp段合法性
        //判断截取开始位置
        int start=msg.indexOf("CP=&&");
        if (start==-1){
            //报文不包含CP=&&
            throw new AnalysisException(1000,"解析报文出错，报文不包含CP=&&字符");
        }
        if (!msg.endsWith("&&")){
            //报文cp段不是以&&结尾
            throw new AnalysisException(1001,"解析报文出错，报文cp段不是以&&结尾");
        }
        //判断结束位置
        int end=msg.lastIndexOf("&&");
        if (end<(start+5)){
            //报文cp段不是以&&包围
            throw new AnalysisException(2701,"解析报文出错，报文cp段不是以&&包围");
        }
        //截取CP

        String cp=msg.substring(start+5,end);
        if ("".equals(cp)){
            //cp无值
            m.setCP(cpMap);
        }else{
            //解析cp
            Map map = msgToKV(cp);
            m.setCP(map);

        }
        //解析除cp段的其他值
        msg=msg.substring(0,start);
        Map<String,String> map = msgToKV(msg);
        //
        m.setCN(map.get("CN"));
        m.setMN(map.get("MN"));
        m.setPW(map.get("PW"));
        m.setQN(map.get("QN"));
        //添加flag
        m.setFlag(map.get("Flag"));
        //添加ST
        m.setST(map.get("ST"));

        //以分号分隔
        return m;
    }
    public static String exeRtnSuccess(Msg msg) {
        SendMsg sendMsg = new SendMsg();
        sendMsg.setCN(msg.getCN());
        sendMsg.setFlag("0");
        sendMsg.setPW(msg.getPW());
        sendMsg.setMN(msg.getMN());
        sendMsg.setCP(Lists.newArrayList(MapUtil.of("ExeRtn", "1")));
        return KvToMsg(sendMsg, ";");
    }

    public static Map<String,String> msgToKV(String s) throws AnalysisException {
        Map resultMap=new HashMap();
       //以；分隔截取成数组
       String[] cpArry = s.split(";");
       for (String ss:cpArry){
           //以逗号分隔
           if(!ss.startsWith("rawData")){
               String[] c = ss.split(",");
               for (String s1:c){
                   //此处已经是单值 xxx=xxx
                   String[] split = s1.split("=");
                   String key="";
                   String value="";
                   key=split[0];
                   if (split.length==2){
                       //不是key=value的格式
//                   throw new AnalysisException(2701,"解析报文出错，报文cp段的值不是key=value形式");
                       value=split[1];
                   }
                   //特殊处理 ##1234QN=20180210154956000
                   if(key.endsWith("QN")&&!"QN".equals(key.split("QN")[0]))
                       key="QN";
                   resultMap.put(key,value);
               }
           }else {
               String[] split = ss.split("=");
               resultMap.put(split[0],split[1]);
           }

       }
       return resultMap;
   }

    public static String KvToMsg(SendMsg sendMsg, String t) {
        //sendmsg的tostring方法被重写返回除去cp段的字符串
        String s = sendMsg.toString();
        List<Map<String, Object>> cp = sendMsg.getCP();
        if (cp==null||cp.size()==0){
            s+="&&";
        }else{
            for (Map<String, Object> m:cp){
                String maptomsg = maptomsg(m,t);
                s+=maptomsg;
            }
            s=s.substring(0,s.length()-1);
            s+="&&";
        }

        //获取数据段长度
        int length = s.length()+14;
        //前面补0共计4位

        String format = String.format("%04d", length);
        //获取crc校验
        s=format+s;
        int checkCrc = HelperUtil.getCheckCrc(s.getBytes());
        String crc = String.format("%04x",checkCrc).toUpperCase();
        return "##"+s+crc+"\r\n";
    }

    public  static String maptomsg(Map map,String t){
        String s="";
        Set set = map.keySet();
        for (Object k:set){
            String kk=(String)k;
            if (kk=="mn"||"mn".equals(kk)){
                continue;
            }
            String v = (String) map.get(k);

            s+=kk+"="+v+t;
        }
        if (StringUtils.isNotEmpty(s)){
            return s.substring(0,s.length()-1)+";";
        }
        //随便多拼一个字符,防止substring丢失一个字符
      return ";";

    }

    public static void main(String[] args) {
        SendMsg sendMsg=new SendMsg();
        sendMsg.setCN("1003");
        sendMsg.setPW("123456");//暂时写死
        sendMsg.setMN("10488888812345");
        List<Map<String,Object>> CP=new ArrayList<>();
        Map<String,Object> methodparams=new HashMap<>();
        methodparams.put("mn","10488888812345");
        methodparams.put("aaa","2");
        CP.add(methodparams);
        sendMsg.setCP(CP);
        //编码报文
        String s = AnalysisUtil.KvToMsg(sendMsg,";");//第二参数规定报文编码时cp段以什么分隔
        System.out.println(s);
        //String a="##FL=0090;QN=201450800121333000;CN=1001;PW=123456;MN=10488888812345;CP=&&Status=1111&&7EC0\r\n";

//        String a="##FL=0078;QN=20200818155859920;CN=1003;PW=123456;MN=test2020081601;CP=&&ExeRtn=0&&9AC1";
//        System.out.println(a.length());
    }
    public  static class Msg{
        @JSONField(name = "QN")
        private String QN;
        @JSONField(name = "ST")
        private String ST;
        @JSONField(name = "PW")
        private String PW;
        @JSONField(name = "PW")
        private String MN;
        @JSONField(name = "Flag")
        private String Flag;
        @JSONField(name = "CN")
        private String CN;
        @JSONField(name = "CP")
        private Map CP;

        public String getQN() {
            return QN;
        }

        public void setQN(String QN) {
            this.QN = QN;
        }

        public String getST() {
            return ST;
        }

        public void setST(String ST) {
            this.ST = ST;
        }

        public String getPW() {
            return PW;
        }

        public void setPW(String PW) {
            this.PW = PW;
        }

        public String getMN() {
            return MN;
        }

        public void setMN(String MN) {
            this.MN = MN;
        }

        public String getFlag() {
            return Flag;
        }

        public void setFlag(String flag) {
            Flag = flag;
        }

        public String getCN() {
            return CN;
        }

        public void setCN(String CN) {
            this.CN = CN;
        }

        public Map getCP() {
            return CP;
        }

        public void setCP(Map CP) {
            this.CP = CP;
        }
    }
}
